@@ -13,7 +13,7 @@ from __future__ import division
 # compatibility functions and utilities
 from ._utils import _supports_unicode, _environ_cols_wrapper, _range, _unich, \
     _term_move_up, _unicode, WeakSet, _basestring, _OrderedDict, \
-    Comparable, RE_ANSI
+    Comparable, RE_ANSI, _is_ascii, SimpleTextIOWrapper
 from ._monitor import TMonitor
 # native libraries
 import sys
@@ -79,8 +79,8 @@ class TqdmDefaultWriteLock(object):
     an argument to joblib or the parallelism lib you use.
     """
     def __init__(self):
-        # Create global parallelism locks to avoid racing issues with parallel bars
-        # works only if fork available (Linux/MacOSX, but not Windows)
+        # Create global parallelism locks to avoid racing issues with parallel
+        # bars works only if fork available (Linux/MacOSX, but not Windows)
         self.create_mp_lock()
         self.create_th_lock()
         cls = type(self)
@@ -125,6 +125,9 @@ class TqdmDefaultWriteLock(object):
 # context and does not allow the user to use 'spawn' or 'forkserver' methods.
 TqdmDefaultWriteLock.create_th_lock()
 
+ASCII_FMT = " 123456789#"
+UTF_FMT = u" " + u''.join(map(_unich, range(0x258F, 0x2587, -1)))
+
 
 class tqdm(Comparable):
     """
@@ -210,8 +213,8 @@ class tqdm(Comparable):
     @staticmethod
     def ema(x, mu=None, alpha=0.3):
         """
-		Exponential moving average: smoothing to give progressively lower
-		weights to older values.
+        Exponential moving average: smoothing to give progressively lower
+        weights to older values.
 
         Parameters
         ----------
@@ -222,7 +225,7 @@ class tqdm(Comparable):
         alpha  : float, optional
             Smoothing factor in range [0, 1], [default: 0.3].
             Increase to give more weight to recent values.
-			Ranges from 0 (yields mu) to 1 (yields x).
+            Ranges from 0 (yields mu) to 1 (yields x).
         """
         return x if mu is None else (alpha * x) + (1 - alpha) * mu
 
@@ -252,7 +255,7 @@ class tqdm(Comparable):
     @staticmethod
     def format_meter(n, total, elapsed, ncols=None, prefix='', ascii=False,
                      unit='it', unit_scale=False, rate=None, bar_format=None,
-                     postfix=None, unit_divisor=1000):
+                     postfix=None, unit_divisor=1000, **extra_kwargs):
         """
         Return a string-based progress bar given some parameters
 
@@ -274,10 +277,10 @@ class tqdm(Comparable):
         prefix  : str, optional
             Prefix message (included in total width) [default: ''].
             Use as {desc} in bar_format string.
-        ascii  : bool, optional
+        ascii  : bool, optional or str, optional
             If not set, use unicode (smooth blocks) to fill the meter
             [default: False]. The fallback is to use ASCII characters
-            (1-9 #).
+            " 123456789#".
         unit  : str, optional
             The iteration unit [default: 'it'].
         unit_scale  : bool or int or float, optional
@@ -296,7 +299,8 @@ class tqdm(Comparable):
               '{rate_fmt}{postfix}]'
             Possible vars: l_bar, bar, r_bar, n, n_fmt, total, total_fmt,
               percentage, rate, rate_fmt, rate_noinv, rate_noinv_fmt,
-              rate_inv, rate_inv_fmt, elapsed, remaining, desc, postfix.
+              rate_inv, rate_inv_fmt, elapsed, elapsed_s,
+              remaining, remaining_s, desc, postfix, unit.
             Note that a trailing ": " is automatically removed after {desc}
             if the latter is empty.
         postfix  : *, optional
@@ -319,14 +323,14 @@ class tqdm(Comparable):
 
         # apply custom scale if necessary
         if unit_scale and unit_scale not in (True, 1):
-            total *= unit_scale
+            if total:
+                total *= unit_scale
             n *= unit_scale
             if rate:
                 rate *= unit_scale  # by default rate = 1 / self.avg_time
             unit_scale = False
 
-        format_interval = tqdm.format_interval
-        elapsed_str = format_interval(elapsed)
+        elapsed_str = tqdm.format_interval(elapsed)
 
         # if unspecified, attempt to use rate = average speed
         # (we allow manual override since predicting time is an arcane art)
@@ -345,62 +349,59 @@ class tqdm(Comparable):
         if unit_scale:
             n_fmt = format_sizeof(n, divisor=unit_divisor)
             total_fmt = format_sizeof(total, divisor=unit_divisor) \
-                if total else None
+                if total is not None else '?'
         else:
             n_fmt = str(n)
-            total_fmt = str(total)
+            total_fmt = str(total) if total is not None else '?'
 
         try:
             postfix = ', ' + postfix if postfix else ''
         except TypeError:
             pass
 
+        remaining = (total - n) / rate if rate and total else 0
+        remaining_str = tqdm.format_interval(remaining) if rate else '?'
+
+        # format the stats displayed to the left and right sides of the bar
+        if prefix:
+            # old prefix setup work around
+            bool_prefix_colon_already = (prefix[-2:] == ": ")
+            l_bar = prefix if bool_prefix_colon_already else prefix + ": "
+        else:
+            l_bar = ''
+
+        r_bar = '| {0}/{1} [{2}<{3}, {4}{5}]'.format(
+            n_fmt, total_fmt, elapsed_str, remaining_str, rate_fmt, postfix)
+
+        # Custom bar formatting
+        # Populate a dict with all available progress indicators
+        format_dict = dict(
+            n=n, n_fmt=n_fmt, total=total, total_fmt=total_fmt,
+            rate=inv_rate if inv_rate and inv_rate > 1 else rate,
+            rate_fmt=rate_fmt, rate_noinv=rate,
+            rate_noinv_fmt=rate_noinv_fmt, rate_inv=inv_rate,
+            rate_inv_fmt=rate_inv_fmt,
+            elapsed=elapsed_str, elapsed_s=elapsed,
+            remaining=remaining_str, remaining_s=remaining,
+            l_bar=l_bar, r_bar=r_bar,
+            desc=prefix or '', postfix=postfix, unit=unit,
+            # bar=full_bar,  # replaced by procedure below
+            **extra_kwargs)
+
         # total is known: we can predict some stats
         if total:
             # fractional and percentage progress
             frac = n / total
             percentage = frac * 100
 
-            remaining_str = format_interval((total - n) / rate) \
-                if rate else '?'
-
-            # format the stats displayed to the left and right sides of the bar
-            if prefix:
-                # old prefix setup work around
-                bool_prefix_colon_already = (prefix[-2:] == ": ")
-                l_bar = prefix if bool_prefix_colon_already else prefix + ": "
-            else:
-                l_bar = ''
             l_bar += '{0:3.0f}%|'.format(percentage)
-            r_bar = '| {0}/{1} [{2}<{3}, {4}{5}]'.format(
-                n_fmt, total_fmt, elapsed_str, remaining_str, rate_fmt, postfix)
 
             if ncols == 0:
                 return l_bar[:-1] + r_bar[1:]
 
             if bar_format:
-                # Custom bar formatting
-                # Populate a dict with all available progress indicators
-                bar_args = {'n': n,
-                            'n_fmt': n_fmt,
-                            'total': total,
-                            'total_fmt': total_fmt,
-                            'percentage': percentage,
-                            'rate': inv_rate if inv_rate and inv_rate > 1
-                            else rate,
-                            'rate_fmt': rate_fmt,
-                            'rate_noinv': rate,
-                            'rate_noinv_fmt': rate_noinv_fmt,
-                            'rate_inv': inv_rate,
-                            'rate_inv_fmt': rate_inv_fmt,
-                            'elapsed': elapsed_str,
-                            'remaining': remaining_str,
-                            'l_bar': l_bar,
-                            'r_bar': r_bar,
-                            'desc': prefix or '',
-                            'postfix': postfix,
-                            # 'bar': full_bar  # replaced by procedure below
-                            }
+                format_dict.update(l_bar=l_bar, percentage=percentage)
+                # , bar=full_bar  # replaced by procedure below
 
                 # auto-remove colon for empty `desc`
                 if not prefix:
@@ -411,11 +412,11 @@ class tqdm(Comparable):
                     # Format left/right sides of the bar, and format the bar
                     # later in the remaining space (avoid breaking display)
                     l_bar_user, r_bar_user = bar_format.split('{bar}')
-                    l_bar = l_bar_user.format(**bar_args)
-                    r_bar = r_bar_user.format(**bar_args)
+                    l_bar = l_bar_user.format(**format_dict)
+                    r_bar = r_bar_user.format(**format_dict)
                 else:
                     # Else no progress bar, we can just format and return
-                    return bar_format.format(**bar_args)
+                    return bar_format.format(**format_dict)
 
             # Formatting progress bar space available for bar's display
             if ncols:
@@ -424,34 +425,33 @@ class tqdm(Comparable):
                 N_BARS = 10
 
             # format bar depending on availability of unicode/ascii chars
-            if ascii:
-                bar_length, frac_bar_length = divmod(
-                    int(frac * N_BARS * 10), 10)
+            if ascii is True:
+                ascii = ASCII_FMT
+            elif ascii is False:
+                ascii = UTF_FMT
+            nsyms = len(ascii) - 1
+            bar_length, frac_bar_length = divmod(
+                    int(frac * N_BARS * nsyms), nsyms)
 
-                bar = '#' * bar_length
-                frac_bar = chr(48 + frac_bar_length) if frac_bar_length \
-                    else ' '
-
-            else:
-                bar_length, frac_bar_length = divmod(int(frac * N_BARS * 8), 8)
-
-                bar = _unich(0x2588) * bar_length
-                frac_bar = _unich(0x2590 - frac_bar_length) \
-                    if frac_bar_length else ' '
+            bar = ascii[-1] * bar_length
+            frac_bar = ascii[frac_bar_length]
 
             # whitespace padding
             if bar_length < N_BARS:
                 full_bar = bar + frac_bar + \
-                    ' ' * max(N_BARS - bar_length - 1, 0)
+                    ascii[0] * (N_BARS - bar_length - 1)
             else:
                 full_bar = bar + \
-                    ' ' * max(N_BARS - bar_length, 0)
+                    ascii[0] * (N_BARS - bar_length)
 
             # Piece together the bar parts
             return l_bar + full_bar + r_bar
 
-        # no total: no progressbar, ETA, just progress stats
+        elif bar_format:
+            # user-specified bar_format but no total
+            return bar_format.format(bar='?', **format_dict)
         else:
+            # no total: no progressbar, ETA, just progress stats
             return ((prefix + ": ") if prefix else '') + \
                 '{0}{1} [{2}, {3}{4}]'.format(
                     n_fmt, unit, elapsed_str, rate_fmt, postfix)
@@ -710,7 +710,7 @@ class tqdm(Comparable):
                  miniters=None, ascii=None, disable=False, unit='it',
                  unit_scale=False, dynamic_ncols=False, smoothing=0.3,
                  bar_format=None, initial=0, position=None, postfix=None,
-                 unit_divisor=1000, gui=False, **kwargs):
+                 unit_divisor=1000, write_bytes=None, gui=False, **kwargs):
         """
         Parameters
         ----------
@@ -733,7 +733,7 @@ class tqdm(Comparable):
         file  : `io.TextIOWrapper` or `io.StringIO`, optional
             Specifies where to output the progress messages
             (default: sys.stderr). Uses `file.write(str)` and `file.flush()`
-            methods.
+            methods.  For encoding, see `write_bytes`.
         ncols  : int, optional
             The width of the entire output message. If specified,
             dynamically resizes the progressbar to stay within this bound.
@@ -755,9 +755,9 @@ class tqdm(Comparable):
             Tweak this and `mininterval` to get very efficient loops.
             If your progress is erratic with both fast and slow iterations
             (network, skipping items, etc) you should set miniters=1.
-        ascii  : bool, optional
+        ascii  : bool or str, optional
             If unspecified or False, use unicode (smooth blocks) to fill
-            the meter. The fallback is to use ASCII characters `1-9 #`.
+            the meter. The fallback is to use ASCII characters " 123456789#".
         disable  : bool, optional
             Whether to disable the entire progressbar wrapper
             [default: False]. If set to None, disable on non-TTY.
@@ -785,7 +785,8 @@ class tqdm(Comparable):
               '{rate_fmt}{postfix}]'
             Possible vars: l_bar, bar, r_bar, n, n_fmt, total, total_fmt,
               percentage, rate, rate_fmt, rate_noinv, rate_noinv_fmt,
-              rate_inv, rate_inv_fmt, elapsed, remaining, desc, postfix.
+              rate_inv, rate_inv_fmt, elapsed, elapsed_s, remaining,
+              remaining_s, desc, postfix, unit.
             Note that a trailing ": " is automatically removed after {desc}
             if the latter is empty.
         initial  : int, optional
@@ -800,6 +801,10 @@ class tqdm(Comparable):
             Calls `set_postfix(**postfix)` if possible (dict).
         unit_divisor  : float, optional
             [default: 1000], ignored unless `unit_scale` is True.
+        write_bytes  : bool, optional
+            If (default: None) and `file` is unspecified,
+            bytes will be written in Python 2. If `True` will also write
+            bytes. In all other cases will default to unicode.
         gui  : bool, optional
             WARNING: internal parameter - do not use.
             Use tqdm_gui(...) instead. If set, will attempt to use
@@ -809,10 +814,18 @@ class tqdm(Comparable):
         -------
         out  : decorated iterator.
         """
+        if write_bytes is None:
+            write_bytes = file is None and sys.version_info < (3,)
 
         if file is None:
             file = sys.stderr
 
+        if write_bytes:
+            # Despite coercing unicode into bytes, py2 sys.std* streams
+            # should have bytes written to them.
+            file = SimpleTextIOWrapper(
+                file, encoding=getattr(file, 'encoding', 'utf-8'))
+
         if disable is None and hasattr(file, "isatty") and not file.isatty():
             disable = True
 
@@ -838,10 +851,13 @@ class tqdm(Comparable):
             self.disable = True
             self.pos = self._get_free_pos(self)
             self._instances.remove(self)
-            raise (TqdmDeprecationWarning("""\
-`nested` is deprecated and automated. Use position instead for manual control.
-""", fp_write=getattr(file, 'write', sys.stderr.write)) if "nested" in kwargs
-                else TqdmKeyError("Unknown argument(s): " + str(kwargs)))
+            from textwrap import dedent
+            raise (TqdmDeprecationWarning(dedent("""\
+                       `nested` is deprecated and automated.
+                       Use `position` instead for manual control.
+                       """), fp_write=getattr(file, 'write', sys.stderr.write))
+                   if "nested" in kwargs else
+                   TqdmKeyError("Unknown argument(s): " + str(kwargs)))
 
         # Preprocess the arguments
         if ((ncols is None) and (file in (sys.stderr, sys.stdout))) or \
@@ -874,7 +890,7 @@ class tqdm(Comparable):
         if ascii is None:
             ascii = not _supports_unicode(file)
 
-        if bar_format and not ascii:
+        if bar_format and not ((ascii is True) or _is_ascii(ascii)):
             # Convert bar format into unicode since terminal uses unicode
             bar_format = _unicode(bar_format)
 
@@ -926,11 +942,7 @@ class tqdm(Comparable):
             # Initialize the screen printer
             self.sp = self.status_printer(self.fp)
             with self._lock:
-                if self.pos:
-                    self.moveto(abs(self.pos))
-                self.sp(self.__repr__(elapsed=0))
-                if self.pos:
-                    self.moveto(-abs(self.pos))
+                self.display()
 
         # Init the time counter
         self.last_print_t = self._time()
@@ -953,14 +965,23 @@ class tqdm(Comparable):
     def __del__(self):
         self.close()
 
-    def __repr__(self, elapsed=None):
-        return self.format_meter(
-            self.n, self.total,
-            elapsed if elapsed is not None else self._time() - self.start_t,
-            self.dynamic_ncols(self.fp) if self.dynamic_ncols else self.ncols,
-            self.desc, self.ascii, self.unit,
-            self.unit_scale, 1 / self.avg_time if self.avg_time else None,
-            self.bar_format, self.postfix, self.unit_divisor)
+    @property
+    def format_dict(self):
+        """Public API for read-only member access"""
+        return dict(
+            n=self.n, total=self.total,
+            elapsed=self._time() - self.start_t
+            if hasattr(self, 'start_t') else 0,
+            ncols=self.dynamic_ncols(self.fp)
+            if self.dynamic_ncols else self.ncols,
+            prefix=self.desc, ascii=self.ascii, unit=self.unit,
+            unit_scale=self.unit_scale,
+            rate=1 / self.avg_time if self.avg_time else None,
+            bar_format=self.bar_format, postfix=self.postfix,
+            unit_divisor=self.unit_divisor)
+
+    def __repr__(self):
+        return self.format_meter(**self.format_dict)
 
     @property
     def _comparable(self):
@@ -992,12 +1013,11 @@ class tqdm(Comparable):
             avg_time = self.avg_time
             _time = self._time
 
-            try:
-                sp = self.sp
-            except AttributeError:
-                raise TqdmDeprecationWarning("""\
-Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
-""", fp_write=getattr(self.fp, 'write', sys.stderr.write))
+            if not hasattr(self, 'sp'):
+                from textwrap import dedent
+                raise TqdmDeprecationWarning(dedent("""\
+                Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
+                """), fp_write=getattr(self.fp, 'write', sys.stderr.write))
 
             for obj in iterable:
                 yield obj
@@ -1019,12 +1039,7 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
 
                         self.n = n
                         with self._lock:
-                            if self.pos:
-                                self.moveto(abs(self.pos))
-                            # Print bar update
-                            sp(self.__repr__())
-                            if self.pos:
-                                self.moveto(-abs(self.pos))
+                            self.display()
 
                         # If no `miniters` was specified, adjust automatically
                         # to the max iteration rate seen so far between 2 prints
@@ -1098,22 +1113,17 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
                 # EMA (not just overall average)
                 if self.smoothing and delta_t and delta_it:
                     rate = delta_t / delta_it
-                    self.avg_time = self.ema(rate, self.avg_time, self.smoothing)
+                    self.avg_time = self.ema(
+                        rate, self.avg_time, self.smoothing)
 
                 if not hasattr(self, "sp"):
-                    raise TqdmDeprecationWarning("""\
-Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
-""", fp_write=getattr(self.fp, 'write', sys.stderr.write))
+                    from textwrap import dedent
+                    raise TqdmDeprecationWarning(dedent("""\
+                    Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
+                    """), fp_write=getattr(self.fp, 'write', sys.stderr.write))
 
                 with self._lock:
-                    if self.pos:
-                        self.moveto(abs(self.pos))
-
-                    # Print bar update
-                    self.sp(self.__repr__())
-
-                    if self.pos:
-                        self.moveto(-abs(self.pos))
+                    self.display()
 
                 # If no `miniters` was specified, adjust automatically to the
                 # maximum iteration rate seen so far between two prints.
@@ -1171,25 +1181,18 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
             raise  # pragma: no cover
 
         with self._lock:
-            if pos:
-                self.moveto(pos)
-
             if self.leave:
                 if self.last_print_n < self.n:
                     # stats for overall rate (no weighted average)
                     self.avg_time = None
-                    self.sp(self.__repr__())
-                if pos:
-                    self.moveto(-pos)
-                elif not max([abs(getattr(i, "pos", 0))
-                              for i in self._instances] + [0]):
+                    self.display(pos=pos)
+                if not max([abs(getattr(i, "pos", 0))
+                            for i in self._instances] + [pos]):
                     # only if not nested (#477)
                     fp_write('\n')
             else:
-                self.sp('')  # clear up last bar
-                if pos:
-                    self.moveto(-pos)
-                else:
+                self.display(msg='', pos=pos)
+                if not pos:
                     fp_write('\r')
 
     def unpause(self):
@@ -1262,6 +1265,7 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
             self.refresh()
 
     def moveto(self, n):
+        # TODO: private method
         self.fp.write(_unicode('\n' * n + _term_move_up() * -n))
         self.fp.flush()
 
@@ -1290,12 +1294,28 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
 
         if not nolock:
             self._lock.acquire()
-        self.moveto(abs(self.pos))
-        self.sp(self.__repr__())
-        self.moveto(-abs(self.pos))
+        self.display()
         if not nolock:
             self._lock.release()
 
+    def display(self, msg=None, pos=None):
+        """
+        Use `self.sp` and to display `msg` in the specified `pos`.
+
+        Parameters
+        ----------
+        msg  : what to display (default: repr(self))
+        pos  : position to display in. (default: abs(self.pos))
+        """
+        if pos is None:
+            pos = abs(self.pos)
+
+        if pos:
+            self.moveto(pos)
+        self.sp(self.__repr__() if msg is None else msg)
+        if pos:
+            self.moveto(-pos)
+
 
 def trange(*args, **kwargs):
     """
